@@ -10,6 +10,7 @@
 
 import json
 
+from ..json_encoder import CustomJSONEncoder
 from .serializers import SerializerMixin
 
 
@@ -18,8 +19,8 @@ class JSONSerializer(SerializerMixin):
 
     def serialize_object(self, obj, response_ctx=None, *args, **kwargs):
         """Dump the object into a json string."""
-        return json.dumps(obj)
+        return json.dumps(obj, cls=CustomJSONEncoder)
 
     def serialize_object_list(self, obj_list, response_ctx=None, *args, **kwargs):
         """Dump the object list into a json string."""
-        return json.dumps(obj_list)
+        return json.dumps(obj_list, cls=CustomJSONEncoder)
